@@ -295,7 +295,9 @@ class VersionedObject(object):
 
     @classmethod
     def obj_name(cls):
-        """Return a canonical name for this object which will be used over
+        """Return the object's name
+
+        Return a canonical name for this object which will be used over
         the wire for remote hydration.
         """
         return cls.__name__
@@ -518,7 +520,7 @@ class VersionedObject(object):
                      if field.default != obj_fields.UnspecifiedDefault]
 
         for attr in attrs:
-            default = self.fields[attr].default
+            default = copy.deepcopy(self.fields[attr].default)
             if default is obj_fields.UnspecifiedDefault:
                 raise exception.ObjectActionError(
                     action='set_defaults',
@@ -812,6 +814,7 @@ class VersionedObjectSerializer(messaging.NoOpSerializer):
 
     def _process_iterable(self, context, action_fn, values):
         """Process an iterable, taking an action on each value.
+
         :param:context: Request context
         :param:action_fn: Action to take on each item in values
         :param:values: Iterable container of things to take action on
