@@ -56,6 +56,7 @@ class MyObj(base.VersionedObject, base.VersionedObjectDictCompat):
               'rel_object': fields.ObjectField('MyOwnedObject', nullable=True),
               'rel_objects': fields.ListOfObjectsField('MyOwnedObject',
                                                        nullable=True),
+              'mutable_default': fields.ListOfStringsField(default=[]),
               }
 
     @staticmethod
@@ -347,7 +348,9 @@ class _BaseTestCase(test.TestCase):
         self.assertEqual(expected, jsonutils.dumps(obj_val))
 
     def str_comparator(self, expected, obj_val):
-        """Compare an object field to a string in the db by performing
+        """Compare a field to a string value
+
+        Compare an object field to a string in the db by performing
         a simple coercion on the object field value.
         """
         self.assertEqual(expected, str(obj_val))
@@ -365,6 +368,22 @@ class _BaseTestCase(test.TestCase):
 
 
 class TestFixture(_BaseTestCase):
+    def test_fake_indirection_takes_serializer(self):
+        ser = mock.MagicMock()
+        iapi = fixture.FakeIndirectionAPI(ser)
+        ser.serialize_entity.return_value = mock.sentinel.serial
+        iapi.object_action(mock.sentinel.context, mock.sentinel.objinst,
+                           mock.sentinel.objmethod, (), {})
+        ser.serialize_entity.assert_called_once_with(mock.sentinel.context,
+                                                     mock.sentinel.objinst)
+        ser.deserialize_entity.assert_called_once_with(mock.sentinel.context,
+                                                       mock.sentinel.serial)
+
+    def test_indirection_fixture_takes_indirection_api(self):
+        iapi = mock.sentinel.iapi
+        self.useFixture(fixture.IndirectionFixture(iapi))
+        self.assertEqual(iapi, base.VersionedObject.indirection_api)
+
     def test_indirection_action(self):
         self.useFixture(fixture.IndirectionFixture())
         obj = MyObj(context=self.context)
@@ -387,12 +406,22 @@ class TestFixture(_BaseTestCase):
                                                  MyObj.VERSION,
                                                  (), {})
 
+    def test_fake_indirection_serializes_arguments(self):
+        ser = mock.MagicMock()
+        iapi = fixture.FakeIndirectionAPI(serializer=ser)
+        arg1 = mock.MagicMock()
+        arg2 = mock.MagicMock()
+        iapi.object_action(mock.sentinel.context, mock.sentinel.objinst,
+                           mock.sentinel.objmethod, (arg1,), {'foo': arg2})
+        ser.serialize_entity.assert_any_call(mock.sentinel.context, arg1)
+        ser.serialize_entity.assert_any_call(mock.sentinel.context, arg2)
+
     def test_get_hashes(self):
         checker = fixture.ObjectVersionChecker()
         hashes = checker.get_hashes()
         # NOTE(danms): If this object's version or hash changes, this needs
         # to change. Otherwise, leave it alone.
-        self.assertEqual('1.6-69d0534216ba3bcb601e9be3b534fcd5',
+        self.assertEqual('1.6-b56dbb7efe42a7ceb137d958fc4066cf',
                          hashes['TestSubclassedObject'])
 
     def test_test_hashes(self):
@@ -800,7 +829,8 @@ class _TestObject(object):
     def test_object_inheritance(self):
         base_fields = []
         myobj_fields = (['foo', 'bar', 'missing',
-                         'readonly', 'rel_object', 'rel_objects'] +
+                         'readonly', 'rel_object',
+                         'rel_objects', 'mutable_default'] +
                         base_fields)
         myobj3_fields = ['new_field']
         self.assertTrue(issubclass(TestSubclassedObject, MyObj))
@@ -860,9 +890,34 @@ class _TestObject(object):
         self.assertRaises(exception.ReadOnlyFieldError, setattr,
                           obj, 'readonly', 2)
 
+    def test_obj_mutable_default(self):
+        obj = MyObj(context=self.context, foo=123, bar='abc')
+        obj.mutable_default = None
+        obj.mutable_default.append('s1')
+        self.assertEqual(obj.mutable_default, ['s1'])
+
+        obj1 = MyObj(context=self.context, foo=123, bar='abc')
+        obj1.mutable_default = None
+        obj1.mutable_default.append('s2')
+        self.assertEqual(obj1.mutable_default, ['s2'])
+
+    def test_obj_mutable_default_set_default(self):
+        obj1 = MyObj(context=self.context, foo=123, bar='abc')
+        obj1.obj_set_defaults('mutable_default')
+        self.assertEqual(obj1.mutable_default, [])
+        obj1.mutable_default.append('s1')
+        self.assertEqual(obj1.mutable_default, ['s1'])
+
+        obj2 = MyObj(context=self.context, foo=123, bar='abc')
+        obj2.obj_set_defaults('mutable_default')
+        self.assertEqual(obj2.mutable_default, [])
+        obj2.mutable_default.append('s2')
+        self.assertEqual(obj2.mutable_default, ['s2'])
+
     def test_obj_repr(self):
         obj = MyObj(foo=123)
-        self.assertEqual('MyObj(bar=<?>,foo=123,missing=<?>,readonly=<?>,'
+        self.assertEqual('MyObj(bar=<?>,foo=123,missing=<?>,'
+                         'mutable_default=<?>,readonly=<?>,'
                          'rel_object=<?>,rel_objects=<?>)',
                          repr(obj))
 
@@ -980,7 +1035,8 @@ class TestObject(_LocalTest, _TestObject):
     def test_set_all_defaults(self):
         obj = MyObj()
         obj.obj_set_defaults()
-        self.assertEqual(set(['foo']), obj.obj_what_changed())
+        self.assertEqual(set(['mutable_default', 'foo']),
+                         obj.obj_what_changed())
         self.assertEqual(1, obj.foo)
 
     def test_set_defaults_not_overwrite(self):
@@ -1280,6 +1336,23 @@ class TestObjectSerializer(_BaseTestCase):
         self.assertNotIsInstance(obj2, MyNSObj)
         self.assertIn('versioned_object.name', obj2)
 
+    def test_serializer_subclass_base_object_indirection(self):
+        @base.VersionedObjectRegistry.register
+        class MyNSObj(base.VersionedObject):
+            OBJ_SERIAL_NAMESPACE = 'foo'
+            fields = {'foo': fields.IntegerField()}
+            indirection_api = mock.MagicMock()
+
+        class MySerializer(base.VersionedObjectSerializer):
+            OBJ_BASE_CLASS = MyNSObj
+
+        ser = MySerializer()
+        prim = MyNSObj(foo=1).obj_to_primitive()
+        prim['foo.version'] = '2.0'
+        ser.deserialize_entity(mock.sentinel.context, prim)
+        MyNSObj.indirection_api.object_backport.assert_called_once_with(
+            mock.sentinel.context, prim, '1.0')
+
 
 class TestNamespaceCompatibility(test.TestCase):
     def setUp(self):
